//  Enemy.cpp
//  papTermOneCourseWork
//
//  Created by Charles Ringer on 31/10/2015.

#include "Enemy.hpp"

//Constructor
Enemy::Enemy(vector<Wall> &walls):height(20), width(20), speed(1)
{
    //This finds a random place on the map that is not occupied by a wall
    bool done = false;
    //Potentail X and Ys
    int potX, potY;
    
    while(!done)
    {
        //Create a new potential x and y
        potX = 20*((int)ofRandom(1, 39));
        potY = 20*((int)ofRandom(1, 29));
        //If it does not hit any of the walls
        if (!(std::any_of(walls.begin(), walls.end(),
                          [potX, potY](Wall wall)
                          {
                              return potY < wall.getEndY()
                              && potY+20 >= wall.getStartY()
                              && potX < wall.getEndX()
                              && potX+20 > wall.getStartX();
                          })))
        {
            //Set the X and Y vals to these values
            X = potX;
            Y = potY;
            cout << X << ' ' <<  X;
            //And get out of this loop
            done = true;
        }
    }
    //This creates a random starting heading
    dir = (int)ofRandom(0,4);
    
    //This loads the art assets for the enemy
    for (int i = 0; i < 4; i++)
    {
        ofImage temp;
        string filePath = "Assets/Enemies/Zombie/" + std::to_string(i) + ".png";
        temp.allocate(20,20,OF_IMAGE_COLOR);
        temp.loadImage(filePath);
        temp.resize(20,20);
        temp.rotate90(2);
        walking.push_back(temp);
    }

}

void Enemy::disp()
{
    //This is white so the image does not get tinted
    ofSetColor(255);
    //We get the current image
    ofImage dispImg = walking[currentDisp];
    //Rotate it to the direction
    if (dir == 1) dispImg.rotate90(1);
    else if (dir == 2) dispImg.rotate90(2);
    else if (dir == 3) dispImg.rotate90(3);
    //Draw it
    dispImg.draw(X,Y);
    //Then...
    if (ofGetFrameNum() %5==0)
    {
        //Every 5 frames we update the image (making sure to wrap around)
        if (currentDisp <3) currentDisp++;
        else currentDisp = 0;
    }
}

void Enemy::move(vector<Wall> &walls)
{
    //This moves the enemy
    //Switches on dir checking if the enemy can move before moving it and returning.
    // If it can't move it will set the dir to a random direction.
    switch(dir)
    {
    case(0):
        if (canMove('u', walls)) { Y-=speed; return; }
        break;
    case(1):
        if (canMove('r', walls)) { X+=speed; return; }
        break;
    case(2):
        if (canMove('d', walls)) { Y+=speed; return; }
        break;
    case(3):
        if (canMove('l', walls)) { X-=speed; return; }
    }
    dir = (int)ofRandom(0,4);
}

bool Enemy::canMove(char dir, vector<Wall> &walls)
{
    //This takes a direction and returns true if the enemy can move that way
    
    //Each of these blocks does the same but for a differnet direction
    //The any_of in the if startment loops through all walls and returns true if there is a hit
    //Which caused this fucn to return false
    
    if (dir == 'u' && std::any_of(walls.begin(), walls.end(), [this](Wall wall)
        { return Y-1 < wall.getEndY() && Y >= wall.getEndY() &&
        X < wall.getEndX() && X+width > wall.getStartX();}))
        return false;
    
    else if (dir == 'd' && std::any_of(walls.begin(), walls.end(), [this](Wall wall)
        { return Y+height+1 > wall.getStartY() && Y+height <= wall.getStartY() &&
        X < wall.getEndX() && X+width > wall.getStartX();}))
        return false;
    
    else if (dir == 'l' && std::any_of(walls.begin(), walls.end(), [this](Wall wall){return X-1 < wall.getEndX() && X >= wall.getEndX() && Y < wall.getEndY() && Y+width > wall.getStartY();})) return false;
    
    else if (dir == 'r' && std::any_of(walls.begin(), walls.end(), [this](Wall wall){return X+width+1 > wall.getStartX() && X+width <= wall.getStartX() && Y < wall.getEndY() && Y+width > wall.getStartY();})) return false;
    
    //If nothing was hit we can return true
    return true;
}